
package imageConvolution.logic;

import java.awt.image.BufferedImage;
import java.awt.image.Kernel;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;

/**
 * @author TineL
 * @version 1.0, 11.12.2006
 * @since 1.0
 */
public class Convolution {

  public static BufferedImage convolve(final BufferedImage image,
    final float[] convolveMask, boolean normalize, boolean copyEdge) {
    // Raster
    Raster raster=image.getData();
    WritableRaster newRaster=raster.createCompatibleWritableRaster();

    // Mask
    Kernel mask=Mask.createCompatibleKernel(convolveMask);
    float[] maskData=new float[Mask.getMaskLength()];
    maskData=mask.getKernelData(maskData);
    int edgeWidth=mask.getWidth()/2;
    int edgeHeight=mask.getHeight()/2;

    // Normalize factor
    float normalFactor=1;
    float bias=0;
    if (normalize) {
      // Sum up all mask coeficients
      float sum=0;
      for (int mi=0; mi<mask.getWidth()*mask.getHeight(); mi++) {
        sum+=maskData[mi];
      }
      normalFactor=(sum==0)?1:sum;
      bias=(sum==0)?128:0;
    }

    // Convolution
    float[] pix=new float[raster.getNumBands()];
    float[] rgba=new float[raster.getNumBands()];
    for (int iy=0; iy<image.getHeight(); iy++) {
      for (int ix=0; ix<image.getWidth(); ix++) {
        for (int i=0; i<rgba.length; i++)
          rgba[i]=0;

        if (iy<edgeHeight||iy>image.getHeight()-1-edgeHeight||ix<edgeWidth
          ||ix>image.getWidth()-1-edgeWidth) {
          // Edge
          if (copyEdge) {
            rgba=raster.getPixel(ix, iy, rgba);
          } else {
            continue;
          }

        } else {
          // Go through mask coeficients
          for (int my=0; my<mask.getHeight(); my++) {
            for (int mx=0; mx<mask.getWidth(); mx++) {
              pix=raster.getPixel(ix+mx-edgeWidth, iy+my-edgeHeight, pix);
              float m=maskData[mx*mask.getWidth()+my];
              for (int i=0; i<rgba.length; i++)
                rgba[i]+=pix[i]*m;
            }
          }
          // Normalize new pixel
          for (int i=0; i<rgba.length; i++)
            rgba[i]=Math.min(Math.abs(rgba[i]+bias/normalFactor), 255);
        }
        // Set new pixel
        newRaster.setPixel(ix, iy, rgba);
      }
    }

    // Processed image
    BufferedImage newImage=new BufferedImage(image.getColorModel(), newRaster,
      image.isAlphaPremultiplied(), null);
    return newImage;
  }
}
