
package imageConvolution.ui;

import imageConvolution.ui.functions.CopyEdgeFunction;
import imageConvolution.ui.functions.ExitFunction;
import imageConvolution.ui.functions.NormalizeFunction;
import imageConvolution.ui.functions.OpenImageFunction;
import imageConvolution.ui.functions.ProcessFunction;
import imageConvolution.ui.functions.ResetMaskFunction;
import imageConvolution.ui.functions.ResetSourceFunction;
import imageConvolution.ui.functions.SaveImageFunction;
import imageConvolution.ui.functions.SetBlurringFunction;
import imageConvolution.ui.functions.SetEmbossingFunction;
import imageConvolution.ui.functions.SetSharpeningFunction;
import imageConvolution.ui.functions.UseAsSourceFunction;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

/**
 * The main menu for application.
 * 
 * @author TineL
 * @version 1.0, 16.1.2007
 * @since 1.0
 */
public class MainMenu extends JMenuBar {

  // Menus
  private JMenu fileM, editM, convolutionM, maskM;

  // Menu items
  private JMenuItem openMI, saveMI, exitMI;
  private JMenuItem resetMaskMI, setEmbossingMI, setBlurMI, setSharpenMI,
    useAsSourceMI, resetSourceMI, processMI;
  private JCheckBoxMenuItem normalizeMI, copyEdgeMI;

  /**
   * Creates <code>MainMenu</code>.
   * 
   * @since 1.0
   */
  public MainMenu() {
    // File menu
    this.fileM=new JMenu("File");
    getFileM().setMnemonic(KeyEvent.VK_F);

    // File open menu item
    this.openMI=new JMenuItem("Open Image...", KeyEvent.VK_O);
    getOpenMI().setToolTipText("Open an image from disk");
    getOpenMI().setAccelerator(
      KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
    OpenImageFunction.getInstance().addButton(getOpenMI());
    getFileM().add(getOpenMI());

    this.saveMI=new JMenuItem("Save Image As...", KeyEvent.VK_S);
    getSaveMI().setToolTipText("Save processed image to disk");
    getSaveMI().setAccelerator(
      KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
    SaveImageFunction.getInstance().addButton(getSaveMI());
    getFileM().add(getSaveMI());

    getFileM().addSeparator();

    // File exit menu item
    this.exitMI=new JMenuItem("Exit", KeyEvent.VK_X);
    getExitMI().setToolTipText("Exit application");
    getExitMI().setAccelerator(
      KeyStroke.getKeyStroke(KeyEvent.VK_F4, ActionEvent.ALT_MASK));
    ExitFunction.getInstance().addButton(getExitMI());
    getFileM().add(getExitMI());

    add(getFileM());

    this.editM=new JMenu("Edit");
    getEditM().setMnemonic(KeyEvent.VK_E);

    this.convolutionM=new JMenu("Convolution");
    getConvolutionM().setMnemonic(KeyEvent.VK_C);

    this.normalizeMI=new JCheckBoxMenuItem("Normalize");
    getNormalizeMI().setToolTipText(
      "Normalize image average intensity when convolving");
    getNormalizeMI().setMnemonic(KeyEvent.VK_N);
    NormalizeFunction.getInstance().addButton(getNormalizeMI());
    getConvolutionM().add(getNormalizeMI());
    this.copyEdgeMI=new JCheckBoxMenuItem("Copy Edge");
    getCopyEdgeMI().setToolTipText(
      "Just copy image's edges instead of ignoring them when convolving");
    getCopyEdgeMI().setMnemonic(KeyEvent.VK_C);
    CopyEdgeFunction.getInstance().addButton(getCopyEdgeMI());
    getConvolutionM().add(getCopyEdgeMI());

    getEditM().add(getConvolutionM());

    this.maskM=new JMenu("Mask");
    getMaskM().setMnemonic(KeyEvent.VK_M);

    this.resetMaskMI=new JMenuItem("Reset");
    getResetMaskMI().setToolTipText(
      "Set mask to default value - no changes when convolving");
    getResetMaskMI().setMnemonic(KeyEvent.VK_R);
    ResetMaskFunction.getInstance().addButton(getResetMaskMI());
    getMaskM().add(getResetMaskMI());
    this.setEmbossingMI=new JMenuItem("Set to Embossing");
    getSetEmbossingMI().setToolTipText("Set mask to default embossing value");
    getSetEmbossingMI().setMnemonic(KeyEvent.VK_M);
    SetEmbossingFunction.getInstance().addButton(getSetEmbossingMI());
    getMaskM().add(getSetEmbossingMI());
    this.setBlurMI=new JMenuItem("Set to Blurring");
    getSetBlurMI().setToolTipText("Set mask to default blurring value");
    getSetBlurMI().setMnemonic(KeyEvent.VK_B);
    SetBlurringFunction.getInstance().addButton(getSetBlurMI());
    getMaskM().add(getSetBlurMI());
    this.setSharpenMI=new JMenuItem("Set to Sharpening");
    getSetSharpenMI().setToolTipText("Set mask to default sharpening value");
    getSetSharpenMI().setMnemonic(KeyEvent.VK_H);
    SetSharpeningFunction.getInstance().addButton(getSetSharpenMI());
    getMaskM().add(getSetSharpenMI());

    getEditM().add(getMaskM());
    getEditM().addSeparator();

    this.processMI=new JMenuItem("Process Image");
    getProcessMI().setToolTipText(
      "Process source image by convolving with the configured mask");
    getProcessMI().setMnemonic(KeyEvent.VK_P);
    getProcessMI().setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0));
    ProcessFunction.getInstance().addButton(getProcessMI());
    getEditM().add(getProcessMI());

    getEditM().addSeparator();

    this.useAsSourceMI=new JMenuItem("Use Processed Image As Source");
    getUseAsSourceMI().setToolTipText(
      "Use processed image as original to perform additional operation on it");
    getUseAsSourceMI().setMnemonic(KeyEvent.VK_U);
    UseAsSourceFunction.getInstance().addButton(getUseAsSourceMI());
    getEditM().add(getUseAsSourceMI());
    this.resetSourceMI=new JMenuItem("Reset Source Image");
    getResetSourceMI().setToolTipText("Reset the originally opened image");
    getResetSourceMI().setMnemonic(KeyEvent.VK_T);
    ResetSourceFunction.getInstance().addButton(getResetSourceMI());
    getEditM().add(getResetSourceMI());

    add(getEditM());
  }

  /**
   * @return the fileM.
   * @since 1.0
   */
  private JMenu getFileM() {
    return this.fileM;
  }

  /**
   * @return the openMI.
   * @since 1.0
   */
  private JMenuItem getOpenMI() {
    return this.openMI;
  }

  /**
   * @return the exitMI.
   * @since 1.0
   */
  private JMenuItem getExitMI() {
    return this.exitMI;
  }

  /**
   * @return the saveMI.
   * @since 1.0
   */
  private JMenuItem getSaveMI() {
    return this.saveMI;
  }

  /**
   * @return the editM.
   * @since 1.0
   */
  private JMenu getEditM() {
    return this.editM;
  }

  /**
   * @return the convolutionM.
   * @since 1.0
   */
  private JMenu getConvolutionM() {
    return this.convolutionM;
  }

  /**
   * @return the copyEdgeMI.
   * @since 1.0
   */
  private JMenuItem getCopyEdgeMI() {
    return this.copyEdgeMI;
  }

  /**
   * @return the maskM.
   * @since 1.0
   */
  private JMenu getMaskM() {
    return this.maskM;
  }

  /**
   * @return the normalizeMI.
   * @since 1.0
   */
  private JMenuItem getNormalizeMI() {
    return this.normalizeMI;
  }

  /**
   * @return the resetMaskMI.
   * @since 1.0
   */
  private JMenuItem getResetMaskMI() {
    return this.resetMaskMI;
  }

  /**
   * @return the setBlurMI.
   * @since 1.0
   */
  private JMenuItem getSetBlurMI() {
    return this.setBlurMI;
  }

  /**
   * @return the setEmbossingMI.
   * @since 1.0
   */
  private JMenuItem getSetEmbossingMI() {
    return this.setEmbossingMI;
  }

  /**
   * @return the setSharpenMI.
   * @since 1.0
   */
  private JMenuItem getSetSharpenMI() {
    return this.setSharpenMI;
  }

  /**
   * @return the resetSourceMI.
   * @since 1.0
   */
  private JMenuItem getResetSourceMI() {
    return this.resetSourceMI;
  }

  /**
   * @return the useAsSourceMI.
   * @since 1.0
   */
  private JMenuItem getUseAsSourceMI() {
    return this.useAsSourceMI;
  }

  /**
   * @return the processMI.
   * @since 1.0
   */
  private JMenuItem getProcessMI() {
    return this.processMI;
  }
}