
package imageConvolution.ui;

import imageConvolution.ui.utils.Factory;
import imageConvolution.ui.utils.SimpleInternalFrame;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Dimension;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.looks.HeaderStyle;
import com.jgoodies.looks.Options;

/**
 * The application's main window. Holds {@link MainMenu}, {@link StatusBar} and
 * all internal components.
 * 
 * @author TineL
 * @version 1.0, 5.1.2007
 * @since 1.0
 */
public class MainWindow extends JFrame {

  /** The main menu. */
  private MainMenu mainMenu;

  /** The status bar. */
  private StatusBar statusBar;

  private Original originalPane;
  private OperationPanel operationPane;
  private Original changedPane;

  /**
   * Creates <code>MainWindow</code>.
   * 
   * @since 1.0
   */
  public MainWindow() {
    // Add
    setJMenuBar(buildMainMenu());
    getContentPane().add(build());

    // Set
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    setSize(800, 600);
    Dimension screenSize=getToolkit().getScreenSize();
    setLocation((int)(screenSize.getWidth()-getWidth())/2, (int)(screenSize
      .getHeight()-getHeight())/2);
    setExtendedState(JFrame.MAXIMIZED_BOTH);
    setTitle("Image Analysis - Convolution for Embossing, Blurring & Sharpening");
    setVisible(true);
  }

  /**
   * Builds the main menu.
   * 
   * @return the main menu (<code>null</code> not possible).
   * @since 1.0
   */
  private JMenuBar buildMainMenu() {
    this.mainMenu=new MainMenu();
    getMainMenu().putClientProperty(Options.HEADER_STYLE_KEY,
      HeaderStyle.SINGLE);
    return getMainMenu();
  }

  /**
   * Builds subcomponents.
   * 
   * @return the component containing subcomponents (<code>null</code> not
   *         possible).
   * @since 1.0
   */
  private JComponent build() {
    JPanel mainPanel=new JPanel(new BorderLayout());
    mainPanel.setOpaque(false);
    mainPanel.add(buildHorizontalSplit(), BorderLayout.CENTER);

    this.statusBar=new StatusBar();
    getStatusBar().putClientProperty(Options.HEADER_STYLE_KEY,
      HeaderStyle.SINGLE);
    mainPanel.add(getStatusBar(), BorderLayout.SOUTH);

    return mainPanel;
  }

  /**
   * Builds 3 horizontally splitted internal frames.
   * 
   * @return the component containing subcomponents (<code>null</code> not
   *         possible).
   * @since 1.0
   */
  private JComponent buildHorizontalSplit() {
    FormLayout layout=new FormLayout("100px:grow, 3px, pref, 3px, 100px:grow",
      "fill:100px:grow");
    DefaultFormBuilder builder=new DefaultFormBuilder(layout);
    builder.append(buildMainLeftPanel());
    builder.append(buildCenterPanel());
    builder.append(buildMainRightPanel());
    return builder.getPanel();
  }

  /**
   * Builds the pane.
   * 
   * @return the pane (<code>null</code> not possible).
   * @since 1.0
   */
  private JComponent buildMainLeftPanel() {
    this.originalPane=new Original();
    JScrollPane sp=Factory.createStrippedScrollPane(getOriginalPane());
    SimpleInternalFrame sif=new SimpleInternalFrame("Source Image");
    sif.setContent(sp);
    return sif;
  }

  /**
   * Builds the pane.
   * 
   * @return the pane (<code>null</code> not possible).
   * @since 1.0
   */
  private JComponent buildCenterPanel() {
    this.operationPane=new OperationPanel();
    SimpleInternalFrame sif=new SimpleInternalFrame("Operation");
    sif.setContent(operationPane.getPanel());
    return sif;
  }

  /**
   * Builds table.
   * 
   * @return the table (<code>null</code> not possible).
   * @since 1.0
   */
  private JComponent buildMainRightPanel() {
    this.changedPane=new Original();
    JScrollPane sp=Factory.createStrippedScrollPane(getChangedPane());
    SimpleInternalFrame sif=new SimpleInternalFrame("Processed Image");
    sif.setContent(sp);
    return sif;
  }

  public void setWaitCursor(boolean wait) {
    if (wait)
      setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    else
      setCursor(null);
  }

  /**
   * @return the main menu (<code>null</code> not possible).
   * @since 1.3
   */
  MainMenu getMainMenu() {
    return this.mainMenu;
  }

  /**
   * @return the changedPane.
   * @since 1.0
   */
  public Original getChangedPane() {
    return this.changedPane;
  }

  /**
   * @return the originalPane.
   * @since 1.0
   */
  public Original getOriginalPane() {
    return this.originalPane;
  }

  /**
   * @return the operation pane.
   * @since 1.0
   */
  public OperationPanel getOperationPane() {
    return this.operationPane;
  }

  /**
   * @return the status bar (<code>null</code> not possible).
   * @since 1.0
   */
  public StatusBar getStatusBar() {
    return this.statusBar;
  }
}