
package imageConvolution.ui;

import imageConvolution.logic.Mask;
import imageConvolution.ui.functions.CopyEdgeFunction;
import imageConvolution.ui.functions.NormalizeFunction;
import imageConvolution.ui.functions.OpenImageFunction;
import imageConvolution.ui.functions.ProcessFunction;
import imageConvolution.ui.functions.ResetMaskFunction;
import imageConvolution.ui.functions.SaveImageFunction;
import imageConvolution.ui.functions.SetBlurringFunction;
import imageConvolution.ui.functions.SetEmbossingFunction;
import imageConvolution.ui.functions.SetSharpeningFunction;

import java.awt.Dimension;
import java.awt.event.KeyEvent;
import java.text.NumberFormat;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFormattedTextField;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

/**
 * @author TineL
 * @version 1.0, 5.1.2007
 * @since 1.0
 */
public class OperationPanel {

  private JPanel panel;

  private JCheckBox normalizeCB;
  private JCheckBox copyEdgeCB;

  private JFormattedTextField mask00, mask01, mask02, mask10, mask11, mask12,
    mask20, mask21, mask22;

  private JButton openB;
  private JButton saveB;
  private JButton resetB;
  private JButton embossB;
  private JButton blurB;
  private JButton sharpenB;
  private JButton processB;

  public OperationPanel() {
    build();
    setMask(Mask.MASK_NO_OP);
  }

  private void build() {
    this.openB=new JButton("Open Image...");
    getOpenB().setToolTipText("Open an image from disk");
    getOpenB().setMnemonic(KeyEvent.VK_O);
    OpenImageFunction.getInstance().addButton(getOpenB());
    this.saveB=new JButton("Save Image As...");
    getSaveB().setToolTipText("Save processed image to disk");
    getSaveB().setMnemonic(KeyEvent.VK_S);
    SaveImageFunction.getInstance().addButton(getSaveB());

    this.normalizeCB=new JCheckBox("Normalize");
    getNormalizeCB().setToolTipText(
      "Normalize image average intensity when convolving");
    getNormalizeCB().setMnemonic(KeyEvent.VK_N);
    NormalizeFunction.getInstance().addButton(getNormalizeCB());
    this.copyEdgeCB=new JCheckBox("Copy Edge");
    getCopyEdgeCB().setToolTipText(
      "Just copy image's edges instead of ignoring them when convolving");
    getCopyEdgeCB().setMnemonic(KeyEvent.VK_C);
    CopyEdgeFunction.getInstance().addButton(getCopyEdgeCB());

    this.mask00=buildField();
    this.mask01=buildField();
    this.mask02=buildField();
    this.mask10=buildField();
    this.mask11=buildField();
    this.mask12=buildField();
    this.mask20=buildField();
    this.mask21=buildField();
    this.mask22=buildField();

    this.resetB=new JButton("Reset");
    getResetB().setToolTipText(
      "Set mask to default value - no changes when convolving");
    getResetB().setMnemonic(KeyEvent.VK_R);
    ResetMaskFunction.getInstance().addButton(getResetB());
    this.embossB=new JButton("Set to Embossing");
    getEmbossB().setToolTipText("Set mask to default embossing value");
    getEmbossB().setMnemonic(KeyEvent.VK_M);
    SetEmbossingFunction.getInstance().addButton(getEmbossB());
    this.blurB=new JButton("Set to Blurring");
    getBlurB().setToolTipText("Set mask to default blurring value");
    getBlurB().setMnemonic(KeyEvent.VK_B);
    SetBlurringFunction.getInstance().addButton(getBlurB());
    this.sharpenB=new JButton("Set to Sharpening");
    getSharpenB().setToolTipText("Set mask to default sharpening value");
    getSharpenB().setMnemonic(KeyEvent.VK_H);
    SetSharpeningFunction.getInstance().addButton(getSharpenB());

    this.processB=new JButton("Process Image");
    getProcessB().setToolTipText(
      "Process source image by convolving with the configured mask");
    getProcessB().setPreferredSize(new Dimension(10, 40));
    getProcessB().setMnemonic(KeyEvent.VK_P);
    ProcessFunction.getInstance().addButton(getProcessB());

    FormLayout layout=new FormLayout(
      "30dlu:grow, 3dlu, 30dlu:grow, 3dlu, 30dlu:grow", "");
    DefaultFormBuilder builder=new DefaultFormBuilder(layout);
    builder.setDefaultDialogBorder();
    builder.appendSeparator("Open && Save");
    builder.append(getOpenB(), 5);
    builder.nextLine();
    builder.append(getSaveB(), 5);
    builder.nextLine();
    builder.appendSeparator("Convolution");
    builder.append(getNormalizeCB(), 5);
    builder.nextLine();
    builder.append(getCopyEdgeCB(), 5);
    builder.nextLine();
    builder.appendSeparator("Mask");
    builder.append(getMask00());
    builder.append(getMask01());
    builder.append(getMask02());
    builder.nextLine();
    builder.append(getMask10());
    builder.append(getMask11());
    builder.append(getMask12());
    builder.nextLine();
    builder.append(getMask20());
    builder.append(getMask21());
    builder.append(getMask22());
    builder.nextLine();
    builder.append(getResetB(), 5);
    builder.nextLine();
    builder.append(getEmbossB(), 5);
    builder.nextLine();
    builder.append(getBlurB(), 5);
    builder.nextLine();
    builder.append(getSharpenB(), 5);
    builder.nextLine();
    builder.appendSeparator("Process");
    builder.append(getProcessB(), 5);

    this.panel=builder.getPanel();
  }

  private JFormattedTextField buildField() {
    NumberFormat format=NumberFormat.getNumberInstance();
    format.setMinimumFractionDigits(2);
    JFormattedTextField field=new JFormattedTextField(format);
    field.setHorizontalAlignment(JTextField.RIGHT);
    return field;
  }

  public void setMask(float[] mask) {
    if (mask.length!=9)
      throw new IllegalArgumentException("Mask length must be 9!");

    getMask00().setValue(new Float(mask[0]));
    getMask01().setValue(new Float(mask[1]));
    getMask02().setValue(new Float(mask[2]));
    getMask10().setValue(new Float(mask[3]));
    getMask11().setValue(new Float(mask[4]));
    getMask12().setValue(new Float(mask[5]));
    getMask20().setValue(new Float(mask[6]));
    getMask21().setValue(new Float(mask[7]));
    getMask22().setValue(new Float(mask[8]));
  }

  public float[] getMask() {
    float[] mask=new float[9];

    mask[0]=((Number)getMask00().getValue()).floatValue();
    mask[1]=((Number)getMask01().getValue()).floatValue();
    mask[2]=((Number)getMask02().getValue()).floatValue();
    mask[3]=((Number)getMask10().getValue()).floatValue();
    mask[4]=((Number)getMask11().getValue()).floatValue();
    mask[5]=((Number)getMask12().getValue()).floatValue();
    mask[6]=((Number)getMask20().getValue()).floatValue();
    mask[7]=((Number)getMask21().getValue()).floatValue();
    mask[8]=((Number)getMask22().getValue()).floatValue();

    return mask;
  }

  /**
   * @return the panel.
   * @since 1.0
   */
  JPanel getPanel() {
    return this.panel;
  }

  /**
   * @return the normalizeCB.
   * @since 1.0
   */
  private JCheckBox getNormalizeCB() {
    return this.normalizeCB;
  }

  /**
   * @return the copyEdgeCB.
   * @since 1.0
   */
  private JCheckBox getCopyEdgeCB() {
    return this.copyEdgeCB;
  }

  /**
   * @return the mask00.
   * @since 1.0
   */
  private JFormattedTextField getMask00() {
    return this.mask00;
  }

  /**
   * @return the mask01.
   * @since 1.0
   */
  private JFormattedTextField getMask01() {
    return this.mask01;
  }

  /**
   * @return the mask02.
   * @since 1.0
   */
  private JFormattedTextField getMask02() {
    return this.mask02;
  }

  /**
   * @return the mask10.
   * @since 1.0
   */
  private JFormattedTextField getMask10() {
    return this.mask10;
  }

  /**
   * @return the mask11.
   * @since 1.0
   */
  private JFormattedTextField getMask11() {
    return this.mask11;
  }

  /**
   * @return the mask12.
   * @since 1.0
   */
  private JFormattedTextField getMask12() {
    return this.mask12;
  }

  /**
   * @return the mask20.
   * @since 1.0
   */
  private JFormattedTextField getMask20() {
    return this.mask20;
  }

  /**
   * @return the mask21.
   * @since 1.0
   */
  private JFormattedTextField getMask21() {
    return this.mask21;
  }

  /**
   * @return the mask22.
   * @since 1.0
   */
  private JFormattedTextField getMask22() {
    return this.mask22;
  }

  /**
   * @return the blurB.
   * @since 1.0
   */
  private JButton getBlurB() {
    return this.blurB;
  }

  /**
   * @return the embossB.
   * @since 1.0
   */
  private JButton getEmbossB() {
    return this.embossB;
  }

  /**
   * @return the sharpenB.
   * @since 1.0
   */
  private JButton getSharpenB() {
    return this.sharpenB;
  }

  /**
   * @return the processB.
   * @since 1.0
   */
  private JButton getProcessB() {
    return this.processB;
  }

  /**
   * @return the resetB.
   * @since 1.0
   */
  private JButton getResetB() {
    return this.resetB;
  }

  /**
   * @return the openB.
   * @since 1.0
   */
  private JButton getOpenB() {
    return this.openB;
  }

  /**
   * @return the saveB.
   * @since 1.0
   */
  private JButton getSaveB() {
    return this.saveB;
  }
}
