
package imageConvolution.ui;

import java.awt.BorderLayout;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.FormLayout;

/**
 * The status bar in the main window. Shows some information.
 * 
 * @author TineL
 * @version 1.0, 5.1.2007
 * @since 1.0
 */
public class StatusBar extends JPanel {

  /** Left info label. */
  private JLabel leftLabel;

  /** Right info label. */
  private JLabel rightLabel;

  /**
   * Creates <code>StatusBar</code>.
   * 
   * @since 1.0
   */
  public StatusBar() {
    // Set
    setLayout(new BorderLayout());
    Border outside=BorderFactory.createEtchedBorder(EtchedBorder.LOWERED);
    Border inside=Borders.DLU2_BORDER;
    setBorder(BorderFactory.createCompoundBorder(outside, inside));

    // Add
    add(build(), BorderLayout.CENTER);
  }

  /**
   * Builds subcomponents.
   * 
   * @return the component containing subcomponents (<code>null</code> not
   *         possible).
   * @since 1.0
   */
  private JComponent build() {
    this.leftLabel=new JLabel();
    this.rightLabel=new JLabel("Ready");

    FormLayout layout=new FormLayout("left:40dlu:grow, 10dlu, right:pref",
      "pref");
    DefaultFormBuilder builder=new DefaultFormBuilder(layout);
    builder.append(getLeftLabel());
    builder.append(getRightLabel());

    return builder.getPanel();
  }

  /**
   * Sets the file description on left side of the status bar.
   * 
   * @param description the file description (<code>null</code> permitted).
   * @since 1.0
   */
  public void setFileDescription(String description) {
    getLeftLabel().setText(description);
    getLeftLabel().setToolTipText(description);
  }

  /**
   * Sets the process message on the right side of the status bar.
   * 
   * @param message the process message (<code>null</code> permitted).
   * @since 1.0
   */
  public void setProcessMessage(String message) {
    getRightLabel().setText(message);
  }

  /**
   * @return the left info label.
   * @since 1.0
   */
  private JLabel getLeftLabel() {
    return this.leftLabel;
  }

  /**
   * @return the right info label.
   * @since 1.0
   */
  private JLabel getRightLabel() {
    return this.rightLabel;
  }
}
