/*
 * Copyright 2004-2006 Autronic d.o.o. Slovenia
 * 
 * You may not use this file except with explicit written permission
 * of Autronic d.o.o. Slovenia.
 */

package imageConvolution.ui.functions;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractButton;

/**
 * The abstract class that represents the function (the push function)
 * associated to an any type of the Swing {@link javax.swing.AbstractButton}.
 * This class collects all buttons that have the same functionality. The
 * function constist of the action, performed when the button's event occures,
 * and the function's enable state that reflects to the enable state of the
 * buttons.<br>
 * The abstract method <i>run</i> will be automatically called, when an event
 * occures.
 *
 * @author TineL
 * @version 1.4, 16.3.2006
 * @since 1.4
 */
public abstract class ButtonFunction extends Function implements ActionListener {

  /**
   * Associate the button to this function.
   * 
   * @param button the button (<code>null</code> not permitted).
   * @since 1.4
   */
  public void addButton(AbstractButton button) {
    super.addComponent(button);
    button.addActionListener(this);
  }

  /**
   * Remove the button from this function. 
   * 
   * @param button the button (<code>null</code> not permitted).
   * @since 1.4
   */
  public void removeButton(AbstractButton button) {
    button.removeActionListener(this);
    super.removeComponent(button);
  }

  /**
   * @since 1.4
   */
  public void actionPerformed(ActionEvent e) {
    run();
  }
}
